(function ($) {
Drupal.behaviors.qtip = {
  attach: function(context) {

    // Set delay on click to immediate
    if (Drupal.settings.qtip.show_event_type == 'click') {
      show_delay = 1;
    }
    else {
      show_delay = 140; // This is the default qTip value, set for hover links
    }

    // Determine how to position the tooltip
    if (Drupal.settings.qtip.show_speech_bubble_tip) {
      // Set the proper qtip speech-bubble tip position
      if (Drupal.settings.qtip.show_speech_bubble_tip_side) {
        switch_position = Drupal.settings.qtip.tooltip_position.split('_');
        // We do not want to put tooltip on it's side if it is being displayed
        // in the center or side of an element
        if (switch_position[1] == 'center') {
          tip_position = Drupal.settings.qtip.tooltip_position;
        }
        else {
          tip_position = switch_position[1] + ' ' + switch_position[0];
        }
      }
      // Otherwise just set it to the same value as the tooltip
      else {
        tip_position = Drupal.settings.qtip.tooltip_position;
      }
    }
    else {
      tip_position = false;
    }
    
    // Do we want a shadow to show under the tooltip?
    if(Drupal.settings.qtip.show_shadow) {
      style_classes = 'ui-tooltip-shadow ' + Drupal.settings.qtip.color;
    }
    else {
      style_classes = Drupal.settings.qtip.color;
    }
    
    //Do we want a solid tip to display
    if (Drupal.settings.qtip.show_speech_bubble_tip_solid) {
      solid_tip = false;
    }
    else {
      solid_tip = 5;
    }

    $('.qtip-link').each(function() {
      // if there is a title associated with this qtip...
      if ($(this).children('.qtip-header').length) {
        tooltip_title = $(this).children('.qtip-header').html();
      }
      // if there isn't we don't want a blank title area to show on the qtip...
      else {
        tooltip_title = false;
      }
      // jQuery qTip library API
      $(this).qtip({
        content: {
          text: $(this).children('.qtip-tooltip'),
          title: {
            text: tooltip_title
          }
        },
        position: {
          my: Drupal.settings.qtip.tooltip_position, // my = speech bubble position on tooltip
          at: Drupal.settings.qtip.target_position, // at = where on link text tooltip will appear
          adjust: {
            screen: true // Keeps tooltip within visible window ... should this be admin-configurable??
          }
        },
        style: {
          classes: style_classes,
          tip: {
            corner: tip_position, // Position of speech bubble tip...false will not display tip
            border: solid_tip //parseInt(Drupal.settings.qtip.border_width)
//            mimic: 'left center' // Alternative display of speech bubble tip...mimics as if it were in that position
          }
        },
        show: {
          event: Drupal.settings.qtip.show_event_type,
          solo: false,
          delay: show_delay
        },
        hide: {
          event: Drupal.settings.qtip.hide_event_type,
          fixed: true
        }
      })
    })
  }
};
})(jQuery);